@@ -39,15 +39,13 @@
 #include <mail/em-utils.h>
 #include <mail/em-event.h>
 #include <mail/em-folder-tree-model.h>
-#include <shell/evolution-config-control.h>
-#include <shell/e-component-view.h>
 #include <shell/es-event.h>
+#include <shell/e-shell-view.h>
 
 #include <libindicate/server.h>
 #include <libindicate/indicator.h>
 #include <libindicate/indicator-messages.h>
 
-#include "e-shell.h"
 #include "xutils.h"
 
 #define CONF_DIR          "/apps/evolution/eplugin/evolution_indicator"
@@ -66,7 +64,6 @@
 #define  USER_BLACKLIST_DIR      "indicators/messages/applications-blacklist"
 #define  USER_BLACKLIST_FILENAME "evolution"
 
-static EShell       *evo_shell   = NULL;
 static GStaticMutex  mlock       = G_STATIC_MUTEX_INIT;
 static GConfClient  *client      = NULL;
 
@@ -95,10 +92,9 @@ static gint message_count = 0;
 
 void org_gnome_mail_new_notify    (EPlugin *ep, EMEventTargetFolder *t);
 void org_gnome_mail_read_notify   (EPlugin *ep, EMEventTargetMessage *t);
-void org_gnome_shell_started_done (EPlugin *ep, ESEventTargetShell *t);
 
-int         e_plugin_lib_enable               (EPluginLib *ep, int enable);
-GtkWidget * e_plugin_lib_get_configure_widget (EPlugin *epl);
+int         e_plugin_lib_enable               (EPlugin *ep, int enable);
+GtkWidget * e_plugin_lib_get_configure_widget (EPlugin *ep);
 
 static void show_evolution (gpointer arg0, guint timestamp, gpointer arg1);
 
@@ -143,6 +139,9 @@ evolution_is_focused (void)
   static GdkWindow *root = NULL;
   gboolean          res = FALSE;
   Window            xwindow;
+  GList *list;
+  EShell *evo_shell;
+  EShellWindow *shell_window;
   
   /* Try and do a match through X, by grabbing the current active window and
      checking to see if it's an evolution window */
@@ -173,22 +172,24 @@ evolution_is_focused (void)
     g_free (res_name);
   }
 
-  if (evo_shell)
-  {
-    GList *w;
-    
-    for (w = evo_shell->priv->windows; w; w = w->next)
-    {
-      GtkWindow *window = w->data;
+  evo_shell = e_shell_get_default ();
+  list = e_shell_get_watched_windows (evo_shell);
 
-      if (GTK_IS_WINDOW (window))
+  /* Find the first EShellWindow in the list. */
+  while (list != NULL && !E_IS_SHELL_WINDOW (list->data))
+    list = g_list_next (list);
+
+  g_return_if_fail (list != NULL);
+
+  /* Get the shell window. */
+  shell_window = E_SHELL_WINDOW (list->data);
+
+  if (GTK_IS_WINDOW (shell_window))
+  {
+      if (gtk_window_is_active (GTK_WINDOW (shell_window)))
       {
-        if (gtk_window_is_active (GTK_WINDOW (window)))
-        {
-          return TRUE;
-        }
+        return TRUE;
       }
-    }
   }
   return FALSE;
 }
@@ -220,7 +221,7 @@ void
 org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
 {
 	g_return_if_fail (t != NULL);
-	
+
   if (!t->new)
 		return;
 
@@ -338,10 +339,10 @@ org_gnome_mail_new_notify (EPlugin *ep, EMEventTargetFolder *t)
 void
 org_gnome_mail_read_notify (EPlugin *ep, EMEventTargetMessage *t)
 {
-  GSList *i;
-  
   g_return_if_fail (t != NULL);
 
+  GSList *i;
+
 	g_static_mutex_lock (&mlock);
 
   g_debug ("EI: mail_read_notify");
@@ -685,7 +686,7 @@ command_item_activate (DbusmenuMenuitem * mi, guint timestamp, gpointer user_dat
 }
 
 int
-e_plugin_lib_enable (EPluginLib *ep, int enable)
+e_plugin_lib_enable (EPlugin *ep, int enable)
 {
   is_active = enable;
 
@@ -789,7 +790,8 @@ e_plugin_lib_enable (EPluginLib *ep, int enable)
     gconf_client_notify_remove (client, show_count_id);
     gconf_client_notify_remove (client, accounts_id);
 
-    g_object_unref (client);      client = NULL;
+    g_object_unref (client);
+    client = NULL;
     
     /* Free indicators */
     g_slist_foreach (indicators, (GFunc)g_object_unref, NULL);
@@ -811,33 +813,12 @@ e_plugin_lib_enable (EPluginLib *ep, int enable)
 }
 
 GtkWidget *
-e_plugin_lib_get_configure_widget (EPlugin *epl)
+e_plugin_lib_get_configure_widget (EPlugin *ep)
 {
   g_debug ("EI: Get Configure Widget");
 	return get_cfg_widget ();
 }
 
-/*
- *
- * THIS IS A MAJOR HACK -- DO NOT EMAIL ME ABOUT HOW MAJOR OF A HACK IT REALLY
- * IS. I'M ALREADY LOSING SLEEP OVER IT.
- *
- * IF YOU HAVE ISSUES WITH THIS CODE, FEEL FREE TO ADD _PROPER_ PLUGIN SUPPORT
- * TO EVOLUTION, OR AT LEAST A DBUS INTERFACE.
- *
- * -- 
- * this message was brought to you by Holiday Inn coffee and a cold
- * Berlin morning 
- *
- */
-void 
-org_gnome_shell_started_done (EPlugin *ep, ESEventTargetShell *t)
-{
-  g_debug ("EI: SHELL STARTUP");
-  
-  evo_shell = t->shell;
-}
-
 static void
 on_combo_changed (GtkComboBox *combo, gpointer null)
 {
@@ -869,7 +850,7 @@ on_show_panel_toggled (GtkToggleButton *button, gpointer null)
 }
 
 GtkWidget *
-org_gnome_get_prefs (EPlugin *epl, EConfigHookItemFactoryData *data)
+org_gnome_get_prefs (EPlugin *ep, EConfigHookItemFactoryData *data)
 {
   GtkWidget *vbox, *frame=NULL, *check;
   
@@ -978,44 +959,40 @@ static void
 show_evolution (gpointer arg0, guint timestamp, gpointer arg1)
 {
 #define MAIL_ICON "evolution-mail"
-  EShell *shell = evo_shell;
+  EShell *shell = e_shell_get_default ();
 
   g_debug ("Showing Evolution to user");
 
   if (shell)
   {
     GSList    *i;
-    GList     *w;
+    GList     *list;
     GtkWindow *mail_window = NULL;
+    EShellWindow *shell_window;
+    EShellView *shell_view;
+    GtkAction *action;
 
-    for (w = shell->priv->windows; w; w = w->next)
+    list = e_shell_get_watched_windows (shell);
+
+    /* Find the first EShellWindow in the list. */
+    while (list != NULL && !E_IS_SHELL_WINDOW (list->data))
+        list = g_list_next (list);
+
+    g_return_if_fail (list != NULL);
+
+    /* Present the shell window. */
+    shell_window = E_SHELL_WINDOW (list->data);
+    if (!evolution_is_focused ())
     {
-      GtkWindow *window = w->data;
-
-      if (GTK_IS_WINDOW (window))
-      {
-        if (g_strcmp0 (MAIL_ICON, gtk_window_get_icon_name (window)) == 0)
-        {
-          mail_window = window;
-        }
-      }
-      else
-      {
-        g_warning ("No open windows: Wait, we're not a mac :-/");
-        return;
-      }
+        gtk_window_present_with_time (GTK_WINDOW (shell_window), timestamp);
     }
 
-    if (!GTK_IS_WINDOW (mail_window))
-    {
-      if (shell->priv->windows && shell->priv->windows->data)
-        mail_window = shell->priv->windows->data;
-      else
-        return;
-    }
-
-    gtk_window_present_with_time (mail_window, timestamp);
+    /* Switch to the mail view. */
+    shell_view = e_shell_window_get_shell_view (shell_window, "mail");
+    action = e_shell_view_get_action (shell_view);
+    gtk_action_activate (action);
 
+    /* Setup the indicators */
     for (i = indicators; i; i = i->next)
     {
       IndicateIndicator *indicator = i->data;
